@@ -20,6 +20,9 @@ my %Dispatcher = (
     view => [
         qw< view_list view view_param view_exec >
     ],
+    feed => [
+        qw< feed_list feed feed_param feed_exec >
+    ],
     action => [
         qw< action_list action action_param action_exec  >
     ],
@@ -56,7 +59,7 @@ sub init {
 
     if (my $filtered = $OpenResty::Config{'frontend.filtered'}) {
         #warn "HERE!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!";
-        use lib "$FindBin::Bin/../../../openresty-filter-qp/trunk/lib";
+        #use lib "$FindBin::Bin/../../../openresty-filter-qp/trunk/lib";
         require OpenResty::Filter::QP;
         my @accounts = split /\s+/, $filtered;
         for my $account (@accounts) {
